/**
 * Copyright (c) 2021, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { Field, Form } from "@wso2is/form";
import isEmpty from "lodash-es/isEmpty";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import {
    ConnectorPropertyInterface,
    GovernanceConnectorInterface,
    ServerConfigurationsConstants
} from "../../../../features/server-configurations";

/**
 * Interface for Login Attempt Security Configuration Form props.
 */
interface LoginAttemptSecurityConfigurationFormPropsInterface extends TestableComponentInterface {
    /**
     * Connector's initial values.
     */
    initialValues: GovernanceConnectorInterface;
    /**
     * Callback for form submit.
     * @param {any} values - Resolved Form Values.
     */
    onSubmit: (values) => void;
    /**
     * Is readonly.
     */
    readOnly?: boolean;
    /**
     * Whether the connector is enabled using the toggle button.
     */
    isConnectorEnabled?: boolean;
}

const allowedConnectorFields: string[] = [
    ServerConfigurationsConstants.ACCOUNT_LOCK_ENABLE,
    ServerConfigurationsConstants.MAX_FAILED_LOGIN_ATTEMPTS_TO_ACCOUNT_LOCK,
    ServerConfigurationsConstants.ACCOUNT_LOCK_TIME,
    ServerConfigurationsConstants.ACCOUNT_LOCK_TIME_INCREMENT_FACTOR,
    ServerConfigurationsConstants.ACCOUNT_LOCK_INTERNAL_NOTIFICATION_MANAGEMENT
];

/**
 * Login Attempt Security Configuration Form.
 *
 * @param {LoginAttemptSecurityConfigurationFormPropsInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
export const LoginAttemptSecurityConfigurationFrom: FunctionComponent<LoginAttemptSecurityConfigurationFormPropsInterface> = (
    props: LoginAttemptSecurityConfigurationFormPropsInterface
): ReactElement => {

    const {
        initialValues,
        onSubmit,
        readOnly,
        isConnectorEnabled,
        ["data-testid"]: testId
    } = props;

    const { t } = useTranslation();

    const [ initialConnectorValues, setInitialConnectorValues ]
        = useState<Map<string, ConnectorPropertyInterface>>(undefined);

    /**
     * Flattens and resolved form initial values and field metadata.
     */
    useEffect(() => {

        if (isEmpty(initialValues?.properties)) {
            return;
        }

        const resolvedInitialValues: Map<string, ConnectorPropertyInterface>
            = new Map<string, ConnectorPropertyInterface>();

        initialValues.properties.map((property: ConnectorPropertyInterface) => {
            if (allowedConnectorFields.includes(property.name)) {
                resolvedInitialValues.set(property.name, property);
            }
        });
        setInitialConnectorValues(resolvedInitialValues);
    }, [ initialValues ]);

    /**
     * Prepare form values for submitting.
     *
     * @param values - Form values.
     *
     * @return Sanitized form values.
     */
    const getUpdatedConfigurations = (values) => {
        const data: any = {
            "account.lock.handler.On.Failure.Max.Attempts": values.maxFailedAttempts !== undefined
                ? values.maxFailedAttempts
                : initialConnectorValues?.get("account.lock.handler.On.Failure.Max.Attempts").value,
            "account.lock.handler.Time": values.accountLockTime !== undefined
                ? values.accountLockTime
                : initialConnectorValues?.get("account.lock.handler.Time").value,
            "account.lock.handler.login.fail.timeout.ratio": values.accountLockIncrementFactor !== undefined
                ? values.accountLockIncrementFactor
                : initialConnectorValues?.get("account.lock.handler.login.fail.timeout.ratio").value,
            "account.lock.handler.notification.manageInternally": true
        };

        return data;
    };

    return (
        initialConnectorValues ?
            <Form
                onSubmit={ (values) => onSubmit(getUpdatedConfigurations(values)) }
                uncontrolledForm={ false }
            >
                <Field.Input
                    ariaLabel="maxFailedAttempts"
                    inputType="default"
                    name="maxFailedAttempts"
                    type="number"
                    min="0"
                    label={ t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.maxFailedAttempts.label") }
                    required={ false }
                    placeholder={  t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.maxFailedAttempts.placeholder") }
                    value={ initialConnectorValues?.get(
                        ServerConfigurationsConstants.MAX_FAILED_LOGIN_ATTEMPTS_TO_ACCOUNT_LOCK)?.value.toString()
                            || 0 }
                    maxLength={ 50 }
                    minLength={ 1 }
                    disabled={ !isConnectorEnabled }
                    data-testid={ `${testId}-max-failed-attempts` }
                    hint={  t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.maxFailedAttempts.hint") }
                />
                <Field.Input
                    ariaLabel="accountLockTime"
                    inputType="default"
                    name="accountLockTime"
                    type="number"
                    min="0"
                    label={ t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.accountLockTime.label") }
                    required={ false }
                    placeholder={ t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.accountLockTime.placeholder") }
                    value={ initialConnectorValues?.get(
                        ServerConfigurationsConstants.ACCOUNT_LOCK_TIME)?.value.toString()
                    || 0 }
                    maxLength={ 50 }
                    minLength={ 1 }
                    disabled={ !isConnectorEnabled }
                    data-testid={ `${testId}-account-lock-time` }
                    hint={  t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.accountLockTime.hint") }
                />
                <Field.Input
                    ariaLabel="accountLockIncrementFactor"
                    inputType="default"
                    name="accountLockIncrementFactor"
                    type="number"
                    min="0"
                    label={ t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.accountLockIncrementFactor.label") }
                    required={ false }
                    placeholder={ t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.accountLockIncrementFactor.placeholder") }
                    value={ initialConnectorValues?.get(
                        ServerConfigurationsConstants.ACCOUNT_LOCK_TIME_INCREMENT_FACTOR)?.value.toString()
                    || 0 }
                    maxLength={ 50 }
                    minLength={ 1 }
                    disabled={ !isConnectorEnabled }
                    data-testid={ `${testId}-account-lock-increment` }
                    hint={  t("extensions:manage.serverConfigurations.accountSecurity." +
                        "loginAttemptSecurity.form.fields.accountLockIncrementFactor.hint") }
                />
                <Field.Button
                    size="small"
                    buttonType="primary_btn"
                    ariaLabel="Login attempt security update button"
                    name="update-button"
                    data-testid={ `${testId}-submit-button` }
                    disabled={ !isConnectorEnabled }
                    label={ t("common:update") }
                    hidden={ !isConnectorEnabled || readOnly }
                />
            </Form>
            : null
    );
};

/**
 * Default props for the component.
 */
LoginAttemptSecurityConfigurationFrom.defaultProps = {
    "data-testid": "login-attempts-security-edit-form"
};
